from typing import Any, Dict, List

from paddlex_hps_server import BaseTritonPythonModel, schemas, utils


class TritonPythonModel(BaseTritonPythonModel):
    def get_input_model_type(self):
        return schemas.vehicle_attribute_recognition.InferRequest

    def get_result_model_type(self):
        return schemas.vehicle_attribute_recognition.InferResult

    def run(self, input, log_id):
        file_bytes = utils.get_raw_bytes(input.image)
        image = utils.image_bytes_to_array(file_bytes)

        result = list(
            self.pipeline.predict(
                image,
                det_threshold=input.detThreshold,
                cls_threshold=input.clsThreshold,
            )
        )[0]

        vehicles: List[Dict[str, Any]] = []
        for obj in result["boxes"]:
            vehicles.append(
                dict(
                    bbox=obj["coordinate"],
                    attributes=[
                        dict(label=l, score=s)
                        for l, s in zip(obj["labels"], obj["cls_scores"])
                    ],
                    score=obj["det_score"],
                )
            )
        if self.app_config.visualize:
            output_image_base64 = utils.base64_encode(
                utils.image_to_bytes(result.img["res"])
            )
        else:
            output_image_base64 = None

        return schemas.vehicle_attribute_recognition.InferResult(
            vehicles=vehicles, image=output_image_base64
        )
